//
//  BrotherPrinter.swift
//  BMS_SwiftUI_Sample_v4_Basic
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
//  Created by Rob Roy
//  Copyright © 2020 Brother Mobile Solutions. All rights reserved.
//

import Foundation

// This is a "Container" Class used to hold all data relevant to the current printer model selected by user.
// It is used for convenience as a central data store.
// Designed as a Singleton so it can be accessed globally and because it should only be created once.
class BrotherPrinter:ObservableObject {
    
    // Singleton
    static let shared = BrotherPrinter()
    
    //*** Constants
    // Keys for UserDefaults
    let keyCurrentPrinterModel = "Current Model"
    let keyChannelType = "Channel Type"
    let keyChannelInfo = "Channel Info"
    
    //*** Variable properties
    
    // NOTE: @Published allows SwiftUI to redraw any GUI which depends on this property.
    @Published public var model:SupportedModels
    {
        didSet {
            // Save to UserDefaults
            self.saveDefaultPrinterModel(model: model)
            
            // Update everything in this class to be consistent with the new model
            self.updatePrinterModel(newModel: model)
        }
    }
    
    public var channel:BRLMChannel
    {
        didSet {
            // Save to UserDefaults
            self.saveDefaultChannel(channel: channel)
        }
    }
    
    // Utility method to set both model and channel (e.g. from Search) at the same time.
    func setModelAndChannel(newModel:SupportedModels, newChannel:BRLMChannel)
    {
        self.model = newModel
        self.channel = newChannel
    }
    
    // App contains/specifies default PDF/Image/PRN files to print for each model.
    // We set these defaults when the model changes.
    var printFiles = FilesToPrint()
    
    
#if STORE_PRINT_SETTINGS
    // NOTE: We could define a PrintSettings variable here too and update it when model changes.
    // But, I decided to set this up at print time instead for the following reasons:
    // * It's not needed until print-time, so don't need to keep it in memory.
    // * It's not needed for PRN printing; only needed for PDF and Image.
    // * If you want to "step debug" through the "HardCodedSettings" code that creates it, it's easier to do that at print-time.
    //   - Otherwise, user must "search" again to cause the code to be executed.
    //   - If this code runs at app startup, with some versions of Xcode, Apple shuts down the app too quickly.
    public var printSettings:BRLMPrintSettingsProtocol? = nil
#endif
    
    // ***************************************************
    // Init function(s)
    // ***************************************************
    
    private init()
    {
        // Set model and channel for PJ-773 and Wireless Direct, for initial use
        model = .PJ_773
        channel = BRLMChannel(wifiIPAddress: "192.168.118.1")
        
        // setup other class data to be consistent with the default model
        self.updatePrinterModel(newModel: model)
        
        // If model/channel have been saved to UserDefaults, use these instead
        // NOTE: If these exist, the "didSet" will be called on each to perform other changes
        if self.loadDefaultPrinterModel(model: &model)
        {
            _ = self.loadDefaultChannel(channel: &channel)
        }
    } // init
    
    
    // ***************************************************
    // Private Functions
    // ***************************************************
    
    private func updatePrinterModel(newModel:SupportedModels)
    {
        // Set everything to be consistent with the new model
        
        // This app defines default files to print (PDF, IMAGE, and PRN) based on model so the file is appropriate.
        // In this "Basic" sample, we will only allow printing these built-in files.
        printFiles.setDefaultFilesForModel(model: newModel)
        
        // * NOTE: Channel will only be updated as a result of the PrinterSearch.
        // This "Basic" sample does not provide any GUI for setting channel in a different way.
        
#if STORE_PRINT_SETTINGS
        // In this "Basic" sample, set printSettings based on HardCoded values. This is what most ISVs do anyway.
        printSettings = HardCodedSettings().hardCodedSettingsForModel(model:newModel)
#endif
        
    } // updatePrinterModel
    
    private func saveDefaultPrinterModel(model:SupportedModels)
    {
        //*** Save the new printer model to UserDefaults
        let defaults = UserDefaults.standard
        
        // NOTE: We cannot save enum value directly. This wil CRASH!!!
        // So, we must convert to "rawValue" when we save, and convert it back to enum when we load.
        //
        // WARNING: When SupportedModels is an Int type, re-ordering the enums in code and rebuilding app may change the rawValue.
        //  -> So, "loadDefaultPrinterModel" method could result in "model" being DIFFERENT from last model that was "saved".
        //
        // SupportedModels enum is now defined to be of type String instead of Int.
        // So, now this same code saves "rawValue" to UserDefaults as a STRING instead!!
        // This allows us to re-arrange the values in SupportedModels enum WITHOUT changing the rawValue!!!
        defaults.set(model.rawValue, forKey: keyCurrentPrinterModel)
        
    } // saveDefaultPrinterModel
    
    // RETURNS:
    // * true if defaults exist, model was updated
    // * false if defaults don't exist, model was NOT updated
    private func loadDefaultPrinterModel(model:inout SupportedModels) -> Bool
    {
        //*** Load the default printer model from UserDefaults
        let defaults = UserDefaults.standard
          
        // Before trying to load, assure the key exists.
        let keys = defaults.dictionaryRepresentation()
        if keys[keyCurrentPrinterModel] != nil {
            // Since we saved as the enum rawvalue, load it back the same way.
#if old
            // Version 1.0.x defined SupportedModels enum of type "Int".
            // So, UserDefault was saved as an Int. So, must load back as an integer.
            // But, that strategy has potential problems. c.f. WARNING in saveDefaultPrinterModel
            let tmpRawValue = defaults.integer(forKey: keyCurrentPrinterModel)
  
            // convert rawValue back to enum, and default to same value passed in if there's an error.
            model = SupportedModels(rawValue: tmpRawValue) ?? model
#else
            // Version 1.1+ defines SupportedModels enum of type "String"
            // So, UserDefault is now saved as a string. So, must load back as a string instead of integer.
            // We can convert back to enum almost exactly same as before.
            let tmpRawValue = defaults.string(forKey: keyCurrentPrinterModel)

            // convert rawValue back to enum, and default to same value passed in if there's an error.
            // NOTE: If updating from v1.0 to v1.1, the error case will occur so model will remain unchanged.
            // Let's catch this and return false so we don't load the saved channel.
            // To solve, perform a search again as though running the app for 1st time.
            if let tmpModel = SupportedModels(rawValue: tmpRawValue!) {
                model = tmpModel
            }
            else {return false}
#endif
            
            return true
        }
        else {return false}
        
    } // loadDefaultPrinterModel
    
    private func saveDefaultChannel(channel: BRLMChannel)
    {
        //*** Save the new channel to UserDefaults
        let defaults = UserDefaults.standard
        
        // NOTE: Must use "rawValue" for channelType enum. Cannot set enum value to UserDefaults or it will CRASH.
        
        // NOTE: SDK enums are of type Int, unlike the SupportedModels enum we created in this app which now is of type String.
        // c.f. WARNING comments in saveDefaultPrinterModel about why saving/loading using "Int" is potentially dangerous.
        // But, it's fine as long as SDK doesn't re-order enum values between new SDK releases, which could change the "rawValue".
        
        defaults.set(channel.channelType.rawValue, forKey: keyChannelType)
        defaults.set(channel.channelInfo, forKey: keyChannelInfo)
    } // saveDefaultChannel
    
    // RETURNS:
    // * true if defaults exist, channel was updated
    // * false if defaults don't exist, channel was NOT updated
    private func loadDefaultChannel(channel:inout BRLMChannel) -> Bool
    {
        let defaults = UserDefaults.standard
        
        // Before trying to load, assure the keys exist.
        let keys = defaults.dictionaryRepresentation()
        if keys[keyChannelType] != nil && keys[keyChannelInfo] != nil {
            // both keys exist, OK to proceed
            
            // NOTE: BRLMChannel object is immutable.
            // So we need to load each part individually to temp vars and then instantiate a new one.
            var channelType:BRLMChannelType
            var channelInfo:String
            
            // 1. load channelType (as rawValue, same as we saved it)
            // convert rawValue back to enum, and default to same value passed in if there's an error.
            let tmpRawValue = defaults.integer(forKey: keyChannelType)
            channelType = BRLMChannelType(rawValue: tmpRawValue) ?? channel.channelType
            
            // 2. load channelInfo
            channelInfo = defaults.string(forKey: keyChannelInfo) ?? channel.channelInfo
            
            // 3. replace "inout channel" param with a newly instantiated BRLMChannel object
            switch channelType {
            case .wiFi:
                channel = BRLMChannel(wifiIPAddress: channelInfo)
                
            case .bluetoothMFi:
                channel = BRLMChannel(bluetoothSerialNumber: channelInfo)
                
            case .bluetoothLowEnergy:
                channel = BRLMChannel(bleLocalName: channelInfo)
                
                // For some reason, Obj-C enums produce the compiler warning:
                // "Switch covers known cases, but XXX may have additional unknown values".
                // c.f. https://www.avanderlee.com/swift/unknown-default-enums-in-swift/
                // This is same as "default", i.e. removes the warning mentioned above.
                // However, it will produce a new compiler warning in the FUTURE if the obj-c enum in SDK adds a new value.
            @unknown default:
                print("loadDefaultChannel: Unknown channel")
                // leave channel unchanged
                
            } // switch
            
            return true
        }
        else {return false}
        
    } // loadDefaultChannel
    
    // ***************************************************
    // Public Functions
    // ***************************************************
    public func stringFromChannelType(channelType:BRLMChannelType) -> String
    {
        switch channelType {
        case .wiFi:
            return "WiFi"
        case .bluetoothMFi:
            return "Bluetooth"
        case .bluetoothLowEnergy:
            return "BLE"
        default:
            return "Unsupported ChannelType"
        }
    } // stringFromChannelType
    
    
}
