//
//  CircleImage.swift
//  BMS_SwiftUI_Sample_v4_Basic
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
//  Created by Rob Roy
//  Copyright © 2020 Brother Mobile Solutions. All rights reserved.
//

import SwiftUI

struct CircleImage: View {
    var image:Image
    
    var renderingMode = Image.TemplateRenderingMode.original // other option is .template
    var imageFrameSize:CGFloat = 50
    var overlayFrameSize:CGFloat = 75
    var overlayStrokeWidth:CGFloat = 3
    
    init(image:Image)
    {
        self.image = image
    }
    
    init(image:Image,
         renderingMode:Image.TemplateRenderingMode,
         imageFrameSize:CGFloat,
         overlayFrameSize:CGFloat,
         overlayStrokeWidth:CGFloat)
    {
        self.image = image
        self.renderingMode = renderingMode
        self.imageFrameSize = imageFrameSize
        self.overlayFrameSize = overlayFrameSize
        self.overlayStrokeWidth = overlayStrokeWidth
    }
    
    var body: some View {
        self.image
            .renderingMode(renderingMode)
            .resizable()
            .scaledToFit() // equivalent to .aspectRatio(contentMode: .fit)
            .frame(width: imageFrameSize, height: imageFrameSize, alignment: .center)
        // .clipShape(Circle())
            .overlay(Circle()
                .stroke(Color.gray, lineWidth: overlayStrokeWidth)
                .frame(width: overlayFrameSize, height: overlayFrameSize, alignment: .center)
            )
    }
}

#if DEBUG
struct CircleImage_Previews: PreviewProvider {
    
    // GearButton has a lot of white space, which affects how much it scales. GearButton-cropped is better.
    //static let gearImage = "GearButton"
    static let gearImage = "GearButton-cropped"
    
    static var previews: some View {
        
        
        VStack {
            HStack {
                // Use default init for CircleImage
                // This is good in most cases, as long as image doesn't have much white space (GearButton vs GearButton-cropped)
                Spacer()
                CircleImage(image:Image("filetype_pdf"))
                
                Spacer()
                CircleImage(image: Image(gearImage))
                
                Spacer()
                CircleImage(image:Image(systemName: "printer"))
                Spacer()
            }
            Spacer()
            
            HStack {
                // Use default init for CircleImage
                // NOTE: setting frame here only affects position
                Spacer()
                CircleImage(image:Image("filetype_pdf"))
                    .frame(width: 50, height: 50, alignment: .center)
                
                Spacer()
                CircleImage(image:Image(gearImage))
                    .frame(width: 75, height: 75, alignment: .center)
                
                Spacer()
                CircleImage(image:Image(systemName: "printer"))
                    .frame(width: 100, height: 100, alignment: .center)
                Spacer()
            }
            Spacer()
            HStack {
                // Use custom init for CircleImage
                // Notice framesize has an effect.
                // imageFrameSize = overlayFrameSize, not good if little whitespace in image
                Spacer()
                CircleImage(image: Image("filetype_pdf"), renderingMode: .original, imageFrameSize: 100, overlayFrameSize: 100, overlayStrokeWidth: 3)
                
                Spacer()
                CircleImage(image: Image(gearImage),
                            renderingMode: .original, imageFrameSize: 75, overlayFrameSize: 75, overlayStrokeWidth: 3)
                
                Spacer()
                CircleImage(image: Image(systemName: "printer"), renderingMode: .original, imageFrameSize: 50, overlayFrameSize: 50, overlayStrokeWidth: 3)
                Spacer()
            }
            Spacer()
            
            // BEST ONE (when you need flexibility)!!
            HStack {
                
                // Use custom init for CircleImage
                // Notice framesize has an effect.
                // imageFrameSize < overlayFrameSize, allows image to fully fit inside the overlay
                Spacer()
                CircleImage(image: Image("filetype_pdf"), renderingMode: .original, imageFrameSize: 60, overlayFrameSize: 80, overlayStrokeWidth: 5)
                
                Spacer()
                
                CircleImage(image: Image(gearImage),
                            renderingMode: .original, imageFrameSize: 60, overlayFrameSize: 80, overlayStrokeWidth: 5)
                Spacer()
                CircleImage(image: Image(systemName: "printer"), renderingMode: .original, imageFrameSize: 60, overlayFrameSize: 80, overlayStrokeWidth: 5)
                Spacer()
            }
            
            Spacer()
            
            HStack {
                
                // Use custom init for CircleImage
                // Notice that if overlay too large these will overlap
                Spacer()
                CircleImage(image: Image("filetype_pdf"), renderingMode: .original, imageFrameSize: 75, overlayFrameSize: 150, overlayStrokeWidth: 3)
                
                Spacer()
                
                CircleImage(image: Image(gearImage),
                            renderingMode: .original, imageFrameSize: 75, overlayFrameSize: 150, overlayStrokeWidth: 3)
                
                Spacer()
                CircleImage(image: Image(systemName: "printer"), renderingMode: .original, imageFrameSize: 75, overlayFrameSize: 150, overlayStrokeWidth: 3)
                Spacer()
            }
            
            Spacer()
        }
        
    }
}
#endif
