//
//  ContentView.swift
//  BMS_SwiftUI_Sample_v4_Basic
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
//  Created by Rob Roy
//  Copyright © 2020 Brother Mobile Solutions. All rights reserved.
//

import SwiftUI

struct ContentView: View {
    @ObservedObject var brotherPrinter:BrotherPrinter = BrotherPrinter.shared
    @ObservedObject var appSettings:AppSettings = AppSettings.shared
    
    @State private var showPrintErrorAlert:Bool = false
    @State private var printErrorAlertMessage:String? = nil
    @State private var printingInProgress:Bool = false
    
    var body: some View {
        NavigationView {
            VStack(alignment: .center) {
                //*** Buttons for printing, by file type
                HStack {
                    Spacer()
                    
                    // PDF
                    Button(action: self.printPDF) {
                        CircleImage(image:Image("filetype_pdf"))
                    }.disabled(printingInProgress)
                    
                    Spacer()
                    
                    // IMAGE
                    Button(action:self.printImage) {
                        CircleImage(image:Image("filetype_image"))
                        
                    }.disabled(printingInProgress)
                    
                    Spacer()
                    
                    // PRN
                    Button(action:self.printPRN) {
                        CircleImage(image:Image("filetype_prn"))
                        
                    }.disabled(printingInProgress)
                    
                    Spacer()
                    
                }
                .padding()
                
                Spacer()
                
                // ** IMAGE
                // Display image of (only) the image file we will print.
                // Don't show if it's unexpectedly nil.
                if self.brotherPrinter.printFiles.imageToDisplay != nil {
                    Image(uiImage: self.brotherPrinter.printFiles.imageToDisplay!)
                        .resizable()
                        .aspectRatio(contentMode: .fit)
                        .border(Color.gray, width:1)
                        .padding()
                }
                else { //else the image to display is nil. What todo?
                    /*
                     // NOTE: It's OK to display NO image here in case of an error with the image.
                     // The Spacer() pushes the next set of buttons to the bottom anyway.
                     //
                     // This was an attempt to display something here to indicate there's an error that
                     // shouldn't fail. But, it's not really necessary.
                     
                     //Image("filetype_image") // using an image from our Assets
                     Image(systemName: "questionmark") // using an SF Symbol
                     .resizable()
                     .aspectRatio(contentMode: .fit)
                     .border(Color.gray, width:1)
                     .padding()
                     */
                }
                
                Spacer()
                
                //** Buttons for Printer and App Setttings
                
                HStack {
                    Spacer()
                    
                    // Printer Search Button
                    NavigationLink(destination: PrinterSearchTopView()) {
                        CircleImage(image:Image(systemName: "magnifyingglass"))
                    }
                    
                    Spacer()
                    
                    // AppSettings
                    NavigationLink(destination: AppSettingsView()) {
                        CircleImage(image:Image("GearButton-cropped"))
                    }.padding()
                    
                    Spacer()
                    
                } // HStack
                
                Spacer()
                
                // ** Info about current printer and channel
                HStack {
                    VStack(alignment: .leading) {
                        Text("Printer: ")
                        Text("Channel Type: ")
                        Text("Channel Info: ")
                    }
                    .padding([.leading])
                    
                    let channelTypeString = self.brotherPrinter.stringFromChannelType(channelType: self.brotherPrinter.channel.channelType)
                    
                    VStack(alignment: .leading) {
                        Text("\(self.brotherPrinter.model.title)")
                        Text("\(channelTypeString)")
                        Text("\(self.brotherPrinter.channel.channelInfo)")
                    }
                    
                    Spacer()
                    
                }
                
            } // VStack
            .padding(.vertical)
            
            //* set view background color
            .background(Color.yellow)
            
            //* set Nav Bar Title
            .navigationBarTitle(Text("SDK SwiftUI Sample - Basic"), displayMode: .inline)
            
            //* add Info button for About Box
            .navigationBarItems(
                trailing: NavigationLink(destination: AboutView()) {
                    Image(systemName: "info.circle")
                        .imageScale(.large)
                }
            )
            
        } // NavigationView
        // NOTE: This makes iPad behave same as iPhone, so we don't have the split view.
        // For now, not doing anything different for iPad.
        // NOTE: Checking UIRequiresFullScreen option in project settings doesn't work.
        .navigationViewStyle(StackNavigationViewStyle())
        
        // Alert if error
        .alert(isPresented: $showPrintErrorAlert, content:{ () -> Alert in
            Alert(title: Text("Error"), message: Text(printErrorAlertMessage!))
        })
        
    } // body
    
    //**********************************************************************
    // MARK: Print functions
    //**********************************************************************
    
    func printPDF() -> Void {
        print("printPDF")
        
        // *** Setup PrintSettings to pass to our printing API
        // PDF printing APIs in SDK require a BRLMxxPrintSettings object corresponding to the printer model family.
        // In this sample, we will "hard-code" these settings to specific values, rather than provide a GUI to allow
        // user to choose them dynamically.
        guard let printSettings = HardCodedSettings().hardCodedSettingsForModel(model: self.brotherPrinter.model)
        else {
            self.printErrorAlertMessage = "printPDF ERROR: printSettings is nil"
            self.showPrintErrorAlert = true
            return
        }
        
        //*** update GUI on main thread (i.e. init our @State vars)
        // DISABLE button presses that activate printing while printing is in progress
        self.printingInProgress = true
        
        // Init for no error (NOTE: showPrintErrorAlert should already be false so don't need to set it and redraw the GUI)
        self.printErrorAlertMessage = nil
        
        //*** do printing in a background thread
        let queue = DispatchQueue(label: "printPDF")
        
        queue.async {
            // use local var for convenience
            let fileURL = self.brotherPrinter.printFiles.pdfFilePathURL
            
            if !self.appSettings.printUsingObjC
            {
                // Use Swift
                let pdfPrinter = PDFPrintHandler()
                
                self.printErrorAlertMessage = pdfPrinter.printPDFFilePathURL(pdfFilePathURL: fileURL,
                                                                             channel: self.brotherPrinter.channel,
                                                                             printSettings: printSettings)
            }
            else
            {
                // use ObjC
                let pdfPrinterObjC = ObjCPDFPrintHandler() // demo only, to show to print in ObjC
                
                self.printErrorAlertMessage = pdfPrinterObjC.objcPrintPDFFilePathURL(fileURL,
                                                                                     channel: self.brotherPrinter.channel,
                                                                                     printSettings: printSettings)
            }
            
            // update GUI on main thread
            DispatchQueue.main.async {
                // re-enable print buttons
                self.printingInProgress = false
                
                // if error, enable the alert
                if self.printErrorAlertMessage != nil {
                    self.showPrintErrorAlert = true
                }
            }
        } // queue.async
        
    } // printPDF
    
    
    func printImage() -> Void {
        print("printImage")
        
        // *** Setup PrintSettings to pass to our printing API
        // IMAGE printing APIs in SDK require a BRLMxxPrintSettings object corresponding to the printer model family.
        // In this sample, we will "hard-code" these settings to specific values, rather than provide a GUI to allow
        // user to choose them dynamically.
        guard let printSettings = HardCodedSettings().hardCodedSettingsForModel(model: self.brotherPrinter.model)
        else {
            self.printErrorAlertMessage = "printImage ERROR: printSettings is nil"
            self.showPrintErrorAlert = true
            return
        }
        
        //*** update GUI on main thread (i.e. init our @State vars)
        // DISABLE button presses that activate printing while printing is in progress
        self.printingInProgress = true
        
        // Init for no error (NOTE: showPrintErrorAlert should already be false so don't need to set it and redraw the GUI)
        self.printErrorAlertMessage = nil
        
        //*** do printing in a background thread
        let queue = DispatchQueue(label: "printImage")
        
        queue.async {
            // use local var for convenience
            let fileURL = self.brotherPrinter.printFiles.imageFilePathURL
            
            
            if !self.appSettings.printUsingObjC
            {
                // Use Swift
                let imagePrinter = ImagePrintHandler()
                
                self.printErrorAlertMessage = imagePrinter.printImageURL(imageFilePathURL: fileURL,
                                                                         channel: self.brotherPrinter.channel,
                                                                         printSettings: printSettings)
            } // swift
            else
            {
                // Use ObjC
                let imagePrinterObjC = ObjCImagePrintHandler()
                
                self.printErrorAlertMessage = imagePrinterObjC.objcPrintImageURL(fileURL,
                                                                                 channel: self.brotherPrinter.channel,
                                                                                 printSettings: printSettings)
            } // objc
            
            // update GUI on main thread
            DispatchQueue.main.async {
                // re-enable print buttons
                self.printingInProgress = false
                
                // if error, enable the alert
                if self.printErrorAlertMessage != nil {
                    self.showPrintErrorAlert = true
                }
            }
        } // queue.async
        
    } // printImage
    
    func printPRN() -> Void {
        print("printPRN")
        
        if self.brotherPrinter.model.isPTModel
        {
            self.printErrorAlertMessage = "PRN printing to PT models is not currently supported in this sample."
            self.showPrintErrorAlert = true
            return
        }
        
        //*** update GUI on main thread (i.e. init our @State vars)
        // DISABLE button presses that activate printing while printing is in progress
        self.printingInProgress = true
        
        // Init for no error (NOTE: showPrintErrorAlert should already be false so don't need to set it and redraw the GUI)
        self.printErrorAlertMessage = nil
        
        //*** do printing in a background thread
        
        let queue = DispatchQueue(label: "printPRN")
        
        queue.async {
            
            // use local var for convenience
            let fileURL = self.brotherPrinter.printFiles.prnFilePathURL
            
            // NOTE: printSettings are N/A when printing a PRN file.
            // The PRN file is expected to contain printer-specific data that is independent of the SDK printSettings.
            // Print languages may include ESC/P, ZPL, CPCL, and/or other, depending on the printer model.
            
            if !self.appSettings.printUsingObjC
            {
                // Use Swift
                let prnPrinter = PRNPrintHandler()
                self.printErrorAlertMessage = prnPrinter.printPRNDataURL(prnFilePathURL: fileURL,
                                                                         channel: self.brotherPrinter.channel,
                                                                         model: self.brotherPrinter.model.sdkModel)
            }
            else
            {
                // Use ObjC
                let prnPrinterObjC = ObjCPRNPrintHandler()
                
                self.printErrorAlertMessage = prnPrinterObjC.objcPrintPRNDataURL(fileURL,
                                                                                 channel: self.brotherPrinter.channel,
                                                                                 model: self.brotherPrinter.model.sdkModel)
            }
            
            // update GUI on main thread
            DispatchQueue.main.async {
                // re-enable print buttons
                self.printingInProgress = false
                
                // if error, enable the alert
                if self.printErrorAlertMessage != nil {
                    self.showPrintErrorAlert = true
                }
            }
        } // queue.async
        
    } // printPRN
    
} // struct ContentView


struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        Group {
            ContentView()
        }
    }
}


