//
//  BLESearchResultView.swift
//  BMS_SwiftUI_Sample_v4_Basic
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
//  Created by Rob Roy
//  Copyright © 2020 Brother Mobile Solutions. All rights reserved.
//

import SwiftUI

struct BLESearchResultView: View {
    
    @ObservedObject var brotherPrinter = BrotherPrinter.shared

    //*** Parameters that must be passed in by caller in init

    // Caller must pass in the ARRAY of printer models we want Search to find, as a FILTER.
    // * For a GENERIC search (e.g PrinterSearchTopView), it should contain *ALL* BLE models the app supports.
    // * However, in some cases, you may only want to find a SPECIFIC model. So, the array should contain ONLY
    //   the 1 model you want to find.
    var printersToSearch:[String]
    var doSearchOnAppear:Bool // caller decides this.
    
    // In case our "parent" view wants to display the result, it must define the @State var and pass it in.
    @Binding var bleLocalName:String
    
    //*** Vars that we define in here
    @State var discoveredPrinterList:[BRPtouchDeviceInfo] = [] // list of devices returned by search
    @State var selectedPrinter:BRPtouchDeviceInfo? = nil // used to check row and update printer model
    @State var infoText:String = ""
    @State var isSearching:Bool = false
    
    var body: some View {
        VStack{
            Spacer()
            Text(infoText)
                .background(Color.red)
            
            Spacer()
            List {
                Section(header: Text("Search Result List:"))
                {
                    
                    // NOTE: "id:\.self" is required here if the array does not contain items that are "Identifiable"
                    // This indicates that each item is guaranteed to be unique.
                    // But, suppose there are multiple PJ-773 connected to same network. They all have strPrinterName = "PJ-773". Not unique!
                    // However, as the following article explains, ".self" means the WHOLE object, which will at least have a unique serialNumber.
                    // And, what actually happens is that Swift computes the HASH VALUE of the object to use as the identifier.
                    // So, this should be OK.
                    // https://www.hackingwithswift.com/books/ios-swiftui/why-does-self-work-for-foreach
                    
                    ForEach(discoveredPrinterList, id:\.self) { discoveredPrinter in
                        Button(action: {
                            self.handleRowTap(deviceInfo: discoveredPrinter)
                            
                        })
                        {
                            // NOTE: For BLE, 'strSerialNumber' is NOT being set in DeviceInfo; so can't use this for display or the checkmark.
                            // 'strModelName' will be identical for all printers of same model, so NOT GOOD for checking uniqueness.
                            // Best item to use is 'strPrinterName', which is the "friendly name", and usually includes last 4 digits of the S/N.
                            ChecklistRow (title: discoveredPrinter.strPrinterName,
                                          subtitle: discoveredPrinter.strModelName,
                                          isChecked: self.selectedPrinter?.strPrinterName == discoveredPrinter.strPrinterName)
                            .frame(height:20)
                        } // Button
                    } // ForEach
                } // SectionHeader
            } // List
            .onAppear{
                if self.doSearchOnAppear {self.doSearchByBLE()} // WARNING: This causes crash if we don't dispatch it here or inside this function.
            }
            .navigationBarTitle(Text("Select BLE Printer"), displayMode: .inline)
            .navigationBarItems( // add a search button to NavBar so user can redo the search, disable it while searching
                trailing: Button(action: {self.doSearchByBLE() } ) { Text("Search")}.disabled(self.isSearching)
            )
            // NOTE: backButton will NOT hide UNLESS you add a title and/or item to NavBar. Seems like a bug in SwiftUI.
            // And be sure to update GUI (via @State var) only on main thread, per usual.
            .navigationBarBackButtonHidden(self.isSearching)
            
        } // VStack
    } // body
    
    // get user selected printer fron the list
    func handleRowTap(deviceInfo:BRPtouchDeviceInfo)
    {
        selectedPrinter = deviceInfo
        // NOTE: refer to SDK user guide. BRLMPrinterClassifier is a utility class.
        // This method converts model name into A BRLMPrinterModel
        let sdkmodel:BRLMPrinterModel = BRLMPrinterClassifier.transferEnum(from: deviceInfo.strModelName)
        
        // Convert sdk model to one of our "supported" models.
        // If detected/selected printer is not one that our app supports, this will return nil.
        let selectedModel:SupportedModels? = SupportedModels.supportedModelFromBRLMPrinterModel(model:sdkmodel)
        
        if selectedModel != nil
        {
            // NOTE: I once had app crash here when strBLEAdvertizeLocalName was nil, but strPrinterName wasn't nil at the time.
            // I think they usually both have the same value. Not sure which one should be used in Channel if they are ever different??
            // I believe the LocalName one is what should be used. But, we will now check for nil below to avoid a crash.
            let stringToUse = deviceInfo.strBLEAdvertiseLocalName
            //let stringToUse = deviceInfo.strPrinterName
            
            // The binding var must be set so we can update the BLE Local Name TextField in BLEChannelView
            // during user PrinterSetup, which also uses this same search result view.
            self.bleLocalName = stringToUse == nil ? "" : stringToUse!
            
            // This utility method assures we set these 2 things in the right order.
            self.brotherPrinter.setModelAndChannel(newModel: selectedModel!,
                                                   newChannel: BRLMChannel(bleLocalName: self.bleLocalName))
        }
    } // handleRowTap
    
    func doSearchByBLE()
    {
        guard let bleManager = BRPtouchBLEManager.shared()
        else {
            self.infoText = "ERROR with BLE Manager"
            return
        }
        
        
        // Update the GUI in case we search again using the button
        self.infoText = "Searching, please wait..."
        self.discoveredPrinterList = []
        self.isSearching = true
        
        // IMPORTANT: We MUST dispatch this, otherwise SwiftUI will crash in onAppear if it calls this function.
        // However, if we dispatch to MAIN queue in onAppear above:
        //  * for some reason startSearch return code is false during onAppear
        //  * and Xcode debugger shows an error related to this. I don't yet understand why.
        //  * But, it works fine when we press the Search button in the NavBar. Weird!
        //
        // But, if we dispatch to a BACKGROUND queue here, then it seems to work in BOTH cases.
        
        DispatchQueue.global().async {
            // WARNING: Accessing BLE requires adding the following to info.plist
            // * (iOS 13+) NSBluetoothAlwaysUsageDescription -> $(PRODUCT_NAME) needs access to use Bluetooth
            // * (iOS 6-13) NSBluetoothPeripheralUsageDescription -> $(PRODUCT_NAME) needs access to use Bluetooth
            // NOTE: SwiftUI requires iOS13+ anyway, so only need NSBluetoothAlwaysUsageDescription. But I've added both just in case.
            
            // Start search, using the following competion handler.
            let startOK = bleManager.startSearch { (deviceInfo: BRPtouchDeviceInfo?) in
                // SDK will call this completion handler each time it finds another BLE device, until we call the stopSearch method.
                
                // Let's filter each device returned in this completion handler so we only show devices that are
                // in the "printersToSearch" array passed in.
                if deviceInfo != nil && deviceInfo?.strModelName != nil
                {
                    
                    // NOTE: BLE currently contains "Brother XXX" in DeviceInfo.
                    // * MFi does NOT contain Brother.
                    // * WIFI does contain Brother, but the array provided to SDK WIFI search will work
                    //   whether it contains Brother or not.
                    //
                    // NOTE: The printersToSearch array will contain either
                    // (a) all BLE models array (created in SupportedPrinters)
                    // (b) or it will contain a SINGLE model.
                    // I don't want to have to remember that some channels require
                    // Brother (or not) and build that into the array every time.
                    // So, I decided the best approach is to be CONSISTENT.
                    // i.e. create the model array WITHOUT "Brother " in the name,
                    // then handle here so we check that DeviceInfo string contains the SUBSTRING of the model.
                    //
                    // NOTE: The following only works if both PrintersToSearch array is an EXACT MATCH, meaning array
                    // must add "Brother ".
                    //    if self.printersToSearch.contains(deviceInfo!.strModelName)
                    //
                    // This is better because array only needs the model (without Brother) and we match this as a substring
                    // in the deviceInfo.
                    if self.printersToSearch.contains(where:deviceInfo!.strModelName.contains)
                    {
                        // NOTE: modifying the list causes GUI to update. Only do this on the MAIN thread!
                        DispatchQueue.main.async {
                            print("BLE completion handler: \(deviceInfo!.strPrinterName ?? "invalid deviceInfo")")
                            //print("BLE completion handler: \(deviceInfo!)")
                            
                            // If this device is not already in the list (c.f. bug details in else below where printer is added 2x after sleep),
                            // add the new device to the list.
                            // c.f. https://stackoverflow.com/questions/34502950/checking-if-an-array-of-custom-objects-contain-a-specific-custom-object
                            // I tried simply ".contains(deviceInfo) and it didn't work.
                            // I tried other suggestions in this post related to using "===" which also didn't work.
                            // But, comparing strPrinterName did work.
                            
                            if !self.discoveredPrinterList.contains(where: {$0.strPrinterName == deviceInfo!.strPrinterName}) {
                                self.discoveredPrinterList.append(deviceInfo!)
                            }
                            else {
                                // NOTE: I notice that frequently this completion handler is being called 2x per printer.
                                // But, it doesn't happen all the time. How to reproduce this behavior?
                                // Ahah! it happens AFTER the iPhone goes to sleep. Every time!
                                // * If you try again, it does NOT happen, until iPhone goes to sleep again.
                                // * Restart printer or app doesn't cause it. Only happen on NEXT search AFTER phone sleeps.
                                // * NOTE: Testing with iPhone6s and iOS13...in case this doesn't happen on other devices.
                                
                                print("Removed duplicate deviceInfo")
                            }
                        }
                    }
                }
            } // startSearch
            
            print("StartOK: \(startOK)")
            
            // Allow the search to run for a limited amount of time, then stop it.
            // This allows SDK to call the completion block above when a BLE device is found.
            let delay:Int = 5
            DispatchQueue.main.asyncAfter(deadline: .now() + .seconds(delay)) {
                //this code will be executed only after "delay" seconds have passed
                
                // stop the search
                bleManager.stopSearch()
                
                // update GUI on main thread
                DispatchQueue.main.async {
                    // update GUI
                    self.infoText = "Search completed!"
                    self.isSearching = false
                }
            }
            
        } // DispatchQueue.global().async
        
    } // doSearchByBLE
    
} // struct BLESearchResultView


#if DEBUG
struct BLESearchResultView_Previews: PreviewProvider {
    @State static var bleLocalName = "123456789"
    
    static var previews: some View {
        BLESearchResultView(printersToSearch: SupportedModels.getArrayOfAllSupportedBLEModels(),
                            doSearchOnAppear: true,
                            bleLocalName: $bleLocalName)
    }
}
#endif

