//
//  BluetoothSearchResultView_BRPtouchBluetoothManager.swift
//  BMS_SwiftUI_Sample_v4_Basic
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
//  Created by Rob Roy
//  Copyright © 2020 Brother Mobile Solutions. All rights reserved.
//

import SwiftUI

struct BluetoothSearchResultView_BRPtouchBluetoothManager: View {
    
    @ObservedObject var brotherPrinter = BrotherPrinter.shared
    
    // NOTE: using BRPtouchBluetoothManager, not EAAccessoryManager
    
    //*** Parameters that must be passed in by caller in init
    // Caller must pass in the ARRAY of printer models we want Search to find, as a FILTER.
    // * For a GENERIC search (e.g PrinterSearchTopView), it should contain *ALL* MFi models the app supports.
    // * However, in some cases, you may only want to find a SPECIFIC model. So, the array should contain ONLY
    //   the 1 model you want to find.
    var printersToSearch:[String]
    
    // In case our "parent" view wants to display the result, it must define the @State var and pass it in.
    @Binding var mfiSerialNumber:String
    
    
    // Setup publishers to handle the accessory connect/disconnect notifications
    // We handle this in onReceive below.
    // NOTE: c.f. comments below in stop/startBluetooth functions. It's not necessary to "register" for these notifications.
    // But, it's a good idea to handle them.
    let didConnectNotification = NotificationCenter.default.publisher(for: NSNotification.Name.BRDeviceDidConnect)
    let didDisconnectNotification = NotificationCenter.default.publisher(for: NSNotification.Name.BRDeviceDidDisconnect)
    
    
    @State var discoveredPrinterList:[BRPtouchDeviceInfo] = [] // list of devices returned by search
    @State var selectedPrinter:BRPtouchDeviceInfo? = nil // used to check row and update printer model
    
    var body: some View {
        VStack{
            
            Spacer()
            List {
                Section(header: Text("Connected Printers:"))
                {
                    
                    // NOTE: "id:\.self" is required here if the array does not contain items that are "Identifiable"
                    // This indicates that each item is guaranteed to be unique.
                    // But, suppose there are multiple PJ-773 connected to same network. They all have strPrinterName = "PJ-773". Not unique!
                    // However, as the following article explains, ".self" means the WHOLE object, which will at least have a unique serialNumber.
                    // And, what actually happens is that Swift computes the HASH VALUE of the object to use as the identifier.
                    // So, this should be OK.
                    // https://www.hackingwithswift.com/books/ios-swiftui/why-does-self-work-for-foreach
                    
                    ForEach(discoveredPrinterList, id:\.self) { discoveredPrinter in
                        Button(action: {
                            self.handleRowTap(deviceInfo: discoveredPrinter)
                            
                        })
                        {
                            // NOTE: Best thing to use for uniqueness (for check mark) is the serialNumber.
                            // Also, serial number is easiest thing to use to identify a specific printer among many of same kind,
                            // **EXCEPT** for PJ-763MFi and RJ-4030Ai, since the S/N reported is NOT the S/N of the printer.
                            // "strModelName" will have the complete model name. Though, all models of same kind will be identical.
                            // We could use "strPrinterName", as this contains the Friendly Name which is usually unique since it contains
                            // the last 4 digits of the S/N by default. You might want to do that for PJ-763MFi and RJ-4030Ai.
                            // But, for now, I'll just show S/N and strModelName.
                            
                            ChecklistRow (title: discoveredPrinter.strSerialNumber,
                                          subtitle: discoveredPrinter.strModelName,
                                          isChecked: self.selectedPrinter?.strSerialNumber == discoveredPrinter.strSerialNumber)
                            .frame(height:20)
                        } // Button
                    } // ForEach
                } // Section Header
            } // List
            .onReceive(didConnectNotification, perform: { _ in self.handleConnectNotification()})
            .onReceive(didDisconnectNotification, perform: { _ in self.handleDisconnectNotification()})
            .onAppear{self.startBluetooth() }
            .onDisappear{self.stopBluetooth() }
            .navigationBarTitle(Text("Select Bluetooth Printer (BR)"), displayMode: .inline)
            
            /*
             #if DO_PAIRING
             // add a "pair" button to NavBar so user can search and pair for NEW (unpaired) MFi devices.
             // This is NOT currently working in SwiftUI, per TSI with Apple. Not working as of iOS15.x either.
             // Refer to additional comments below in pairPrintersByBluetoothMFi().
             .navigationBarItems(
             trailing: Button(action: {self.pairPrintersByBluetoothMFi()}) { Text("Pair")}
             )
             #endif
             */
            
        } // VStack
    } // body
    
    // get user selected printer fron the list of connected accessories
    func handleRowTap(deviceInfo:BRPtouchDeviceInfo)
    {
        selectedPrinter = deviceInfo
        
        // NOTE: refer to SDK user guide. BRLMPrinterClassifier is a utility class.
        // This method converts model name into A BRLMPrinterModel
        let sdkmodel:BRLMPrinterModel = BRLMPrinterClassifier.transferEnum(from: deviceInfo.strModelName)
        
        // Convert sdk model to one of our "supported" models.
        // If detected/selected printer is not one that our app supports, this will return nil.
        let selectedModel:SupportedModels? = SupportedModels.supportedModelFromBRLMPrinterModel(model:sdkmodel)
        
        if selectedModel != nil
        {
            // The binding var must be set so we can update the serialNumber TextField in BluetoothChannelView
            // during user PrinterSetup, which also uses this same search result view.
            self.mfiSerialNumber = deviceInfo.strSerialNumber
            
            
            // This utility method assures we set these 2 things in the right order.
            self.brotherPrinter.setModelAndChannel(newModel: selectedModel!,
                                                   newChannel: BRLMChannel(bluetoothSerialNumber:deviceInfo.strSerialNumber))
        }
    } // handleRowTap
    
    func startBluetooth()
    {
#if NOT_NEEDED
        // c.f. comments below in stopBluetooth. Do NOT register for the notifications.
        // It's NOT necessary!! And it causes a bug.
        // Leaving this here for reference.
        //BRPtouchBluetoothManager.shared().registerForBRDeviceNotifications()
#endif
        
        self.updateConnectedAccessories()
    }
    
    func stopBluetooth()
    {
#if NOT_NEEDED
        // WARNING: SDK has bug related to unregister for notifications.
        // After you have called this on the shared manager, ** if device disconnects/reconnects while app is running **,
        // it is NOT possible to print until you shutdown/restart the app.
        // It seems the SDK now will automatically send the connect/disconnect notifications.
        // So, it's not required to register/unregister for them. And, by NOT doing this, you can avoid the bug.
        // So, do NOTHING here!!
        //BRPtouchBluetoothManager.shared().unregisterForBRDeviceNotifications()
#endif
        
    }
    
    // Update "discoveredPrinterList" on view appear or in response to connect/disconnect notifications.
    // Filter the list based on "printersToSearch" array passed in.
    // In some cases, we filter by ALL SupportedModels, in other cases we may only want to show 1 specific model.
    func updateConnectedAccessories()
    {
        // NOTE: Since this gets called in .onAppear, let's dispatch this to remove console warnings saying
        // "UITableView was told to layout its visible cells and other contents without being in the view hierarchy...."
        DispatchQueue.main.async {
            // This will quickly show all currently connected accessories
            
            // Get list of all connected Brother printers (presumably the SDK API will filter to provide only Brother printers, of ANY model)
            let allConnectedDevices:[BRPtouchDeviceInfo] = BRPtouchBluetoothManager.shared().pairedDevices() as! [BRPtouchDeviceInfo]
            
            // Filter the list provided by SDK to match the list of Brother printer models passed in the printersToSearch array.
            var filteredDevices:[BRPtouchDeviceInfo] = []
            for device in allConnectedDevices {
                
                // c.f. comments in BLESearchResultView. The 2nd case will work whether "Brother " is in strModelName or not,
                // as long as we make sure NOT to add "Brother " to the printersToSearch array.
                // The 1st case works too, if we do not add Brother to our array, but it requires an exact match of the array.
                // So, 2nd case is more flexible in case something changes in the future, so it's probably better.
                
                // if self.printersToSearch.contains(device.strModelName)
                if self.printersToSearch.contains(where:device.strModelName.contains)
                {
                    filteredDevices.append(device)
                }
            }
            
            // assign filtered list
            self.discoveredPrinterList = filteredDevices
        }
    } // updateConnectedAccessories
    
    
    func handleConnectNotification()
    {
        print("handleConnectNotification")
        
        self.updateConnectedAccessories()
        
    }
    
    func handleDisconnectNotification()
    {
        print("handleDisconnectNotification")
        
        self.updateConnectedAccessories()
    }
    
    
#if DO_PAIRING
    // In-app Pairing:
    //
    // The brShowBluetoothAccessoryPicker method is supposed to show an Apple-provided picker that allows "in-app Pairing".
    // i.e. a picker will be displayed that shows unpaired devices, allowing user to pair from within your app.
    // Otherwise, the only way to do "pairing" is in iOS Settings -> Bluetooth, i.e. outside your app.
    //
    // ISSUE: Currently, Apple's picker is NOT being displayed in SwiftUI. So, for now, this code has been #if'ed out.
    // Based on other apps we have created, Apple's picker API works correctly in a UIKit implementation.
    // It seems to be an issue specific to SwiftUI.
    //
    // UPDATE 1: I have filed a TSI and BugReport with Apple about this. They have confirmed the bug.
    // Apple said it MAY be fixed in iOS14.5. TBD. (It wasn't).
    //
    // According to Apple support, there's a WORKAROUND which requires using an App-based lifecycle instead of a
    // Scene-based lifecycle, with AppDelegate implemented appropriately.
    // Generally, in this App-based lifecycle scenario, you can use UIHostingController to display a SwiftUI View inside a
    // UIKit-based UIViewController.
    // Since this app is intended to be a SwiftUI demo app, I'm not going to switch to use UIHostingController (for now).
    //
    // If iOS has fixed this in 14.5 or later, we can enable this code. But, we will have to check the OS version in code,
    // or set the deployment target for the iOS version which finally fixes this bug.
    //
    // UPDATE 2: I tested with iOS15.7 and it is still broken with SwiftUI, at least when using a Scene-based Lifecycle.
    // So, if you need to do in-app pairing in a SwiftUI APP, we recommmend that you use the UIHostingController approach
    // as described above. Or, you can use UIKit to create a similar view instead of using SwiftUI.
    // If you require more information or assistance with this, please contact Tech Support.
    
    func pairPrintersByBluetoothMFi()
    {
        // NOTE: Unlike EAAccessoryManager, the BRPtouchBluetoothManager wrapper does NOT have a "completion handler".
        // It's really only used for errors anyway.
        // When user picks a device and pairs successfully, the BRDeviceDidConnect notification should be fired.
        // We are already handling that. c.f. handleConnectNotification().
        BRPtouchBluetoothManager.shared()?.brShowBluetoothAccessoryPicker(withNameFilter: nil)
    }
    
#endif
    
} // struct BluetoothSearchResultView

#if DEBUG
struct BluetoothSearchResultView_BRPtouchBluetoothManager_Previews: PreviewProvider {
    @State static var mfiSerialNumber = "ABC1234567890"
    
    static var previews: some View {
        BluetoothSearchResultView_BRPtouchBluetoothManager(printersToSearch: SupportedModels.getArrayOfAllSupportedMFiModels(),
                                                           mfiSerialNumber: $mfiSerialNumber)
    }
}
#endif

