//
//  BluetoothSearchResultView_EAAccessory.swift
//  BMS_SwiftUI_Sample_v4_Basic
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
//  Created by Rob Roy
//  Copyright © 2020 Brother Mobile Solutions. All rights reserved.
//

import SwiftUI
import ExternalAccessory // EAAccessoryManager

struct BluetoothSearchResultView_EAAccessory: View {
    
    @ObservedObject var brotherPrinter = BrotherPrinter.shared

    // NOTE: using EAAccessoryManager, not BRPtouchBluetoothManager
    
    //*** Parameters that must be passed in by caller in init
    // Caller must pass in the ARRAY of printer models we want Search to find, as a FILTER.
    // * For a GENERIC search (e.g PrinterSearchTopView), it should contain *ALL* MFi models the app supports.
    // * However, in some cases, you may only want to find a SPECIFIC model. So, the array should contain ONLY
    //   the 1 model you want to find.
    var printersToSearch:[String]
    
    // In case our "parent" view wants to display the result, it must define the @State var and pass it in.
    @Binding var mfiSerialNumber:String

    // Setup publishers to handle the accessory connect/disconnect notifications
    // We handle this in onReceive below.
    let didConnectNotification = NotificationCenter.default.publisher(for: NSNotification.Name.EAAccessoryDidConnect)
    let didDisconnectNotification = NotificationCenter.default.publisher(for: NSNotification.Name.EAAccessoryDidDisconnect)
    
    @State var discoveredPrinterList:[EAAccessory] = [] // list of devices returned by search
    @State var selectedPrinter:EAAccessory? = nil // used to check row and update printer model
    
    var body: some View {
        VStack{
            
            Spacer()
            List {
                Section(header: Text("Connected Printers:"))
                {
                    
                    // NOTE: "id:\.self" is required here if the array does not contain items that are "Identifiable"
                    // This indicates that each item is guaranteed to be unique.
                    // But, suppose there are multiple PJ-773 connected to same network. They all have strPrinterName = "PJ-773". Not unique!
                    // However, as the following article explains, ".self" means the WHOLE object, which will at least have a unique serialNumber.
                    // And, what actually happens is that Swift computes the HASH VALUE of the object to use as the identifier.
                    // So, this should be OK.
                    // https://www.hackingwithswift.com/books/ios-swiftui/why-does-self-work-for-foreach
                    
                    ForEach(discoveredPrinterList, id:\.self) { discoveredPrinter in
                        Button(action: {
                            self.handleRowTap(accessory: discoveredPrinter)
                            
                        })
                        {
                            // NOTE: With RJ models (all except RJ-4030Ai), "name" will contain ONLY "RJ".
                            // But, "modelNumber" will have the complete model name. Though, all models of same kind will be identical.
                            // Best thing to use for uniqueness (for check mark) is the serialNumber.
                            ChecklistRow (title: discoveredPrinter.serialNumber,
                                          // subtitle: discoveredPrinter.name,
                                          subtitle: discoveredPrinter.modelNumber,
                                          isChecked: self.selectedPrinter?.serialNumber == discoveredPrinter.serialNumber)
                            .frame(height:20)
                        } // Button
                    } // ForEach
                } // SectionHeader
            } // List
            .onReceive(didConnectNotification, perform: { _ in self.handleConnectNotification()})
            .onReceive(didDisconnectNotification, perform: { _ in self.handleDisconnectNotification()})
            .onAppear{self.startBluetooth() }
            .onDisappear{self.stopBluetooth() }
            .navigationBarTitle(Text("Select Bluetooth Printer (EA)"), displayMode: .inline)
            
            /*
             #if DO_PAIRING
             // add a "pair" button to NavBar so user can search and pair for NEW (unpaired) MFi devices.
             // This is NOT currently working in SwiftUI, per TSI with Apple. Not working as of iOS15.x either.
             // Refer to additional comments below in pairPrintersByBluetoothMFi().
             .navigationBarItems(
             trailing: Button(action: {self.pairPrintersByBluetoothMFi()}) { Text("Pair")}
             )
             #endif
             */
            
        } // VStack
    } // body
    
    // get user selected printer fron the list of connected accessories
    func handleRowTap(accessory:EAAccessory)
    {
        selectedPrinter = accessory
        
        // NOTE: refer to SDK user guide. BRLMPrinterClassifier is a utility class.
        // This method converts model name into A BRLMPrinterModel
        let sdkmodel:BRLMPrinterModel = BRLMPrinterClassifier.transferEnum(from: accessory.modelNumber)
        
        // Convert sdk model to one of our "supported" models.
        // If detected/selected printer is not one that our app supports, this will return nil.
        let selectedModel:SupportedModels? = SupportedModels.supportedModelFromBRLMPrinterModel(model:sdkmodel)
        
        if selectedModel != nil
        {
            // The binding var must be set so we can update the serialNumber TextField in BluetoothChannelView
            // during user PrinterSetup, which also uses this same search result view.
            self.mfiSerialNumber = accessory.serialNumber
            
            
            // This utility method assures we set these 2 things in the right order.
            self.brotherPrinter.setModelAndChannel(newModel: selectedModel!, newChannel: BRLMChannel(bluetoothSerialNumber:accessory.serialNumber))
        }
    } // handleRowTap
    
    func startBluetooth()
    {
        EAAccessoryManager.shared().registerForLocalNotifications()
        self.updateConnectedAccessories()
    }
    
    func stopBluetooth()
    {
        EAAccessoryManager.shared().unregisterForLocalNotifications()
    }
    
    // Update "discoveredPrinterList" on view appear or in response to connect/disconnect notifications.
    // Filter the list based on "printersToSearch" array passed in.
    // In some cases, we filter by ALL SupportedModels, in other cases we may only want to show 1 specific model.
    func updateConnectedAccessories()
    {
        // NOTE: Since this gets called in .onAppear, let's dispatch this to remove console warnings saying
        // "UITableView was told to layout its visible cells and other contents without being in the view hierarchy...."
        DispatchQueue.main.async {
            // This will quickly show all currently connected accessories
            
            // Get list of all connected EAAccessories
            let allConnectedAccessories:[EAAccessory] = EAAccessoryManager.shared().connectedAccessories
            
            // Filter the list provided by EAAccessoryManager to match the list of Brother printer models passed in the printersToSearch array.
            var filteredAccessories:[EAAccessory] = []
            for accessory in allConnectedAccessories {
                // c.f. comments in BLESearchResultView.
                // The 2nd case will work whether "Brother " is in accessory or not,
                // as long as we make sure NOT to add "Brother " to the printersToSearch array.
                // The 1st case works too, if we do not add Brother to our array, but it requires an exact match of the array.
                // So, 2nd case is more flexible in case something changes in the future, so it's probably better.
                
                //if self.printersToSearch.contains(accessory.modelNumber)
                if self.printersToSearch.contains(where:accessory.modelNumber.contains)
                {
                    filteredAccessories.append(accessory)
                }
            }
            
            // assign filtered list
            self.discoveredPrinterList = filteredAccessories
            
        }
        
    } // updateConnectedAccessories
    
    
    func handleConnectNotification()
    {
        print("handleConnectNotification")
        
        self.updateConnectedAccessories()
        
    }
    
    func handleDisconnectNotification()
    {
        print("handleDisconnectNotification")
        
        self.updateConnectedAccessories()
    }
    
    
#if DO_PAIRING
    // In-app Pairing:
    //
    // The showBluetoothAccessoryPicker method is supposed to show an Apple-provided picker that allows "in-app Pairing".
    // i.e. a picker will be displayed that shows unpaired devices, allowing user to pair from within your app.
    // Otherwise, the only way to do "pairing" is in iOS Settings -> Bluetooth, i.e. outside your app.
    //
    // ISSUE: Currently, Apple's picker is NOT being displayed in SwiftUI. So, for now, this code has been #if'ed out.
    // Based on other apps we have created, Apple's picker API works correctly in a UIKit implementation.
    // It seems to be an issue specific to SwiftUI.
    //
    // UPDATE 1: I have filed a TSI and BugReport with Apple about this. They have confirmed the bug.
    // Apple said it MAY be fixed in iOS14.5. TBD. (It wasn't).
    //
    // According to Apple support, there's a WORKAROUND which requires using an App-based lifecycle instead of a
    // Scene-based lifecycle, with AppDelegate implemented appropriately.
    // Generally, in this App-based lifecycle scenario, you can use UIHostingController to display a SwiftUI View inside a
    // UIKit-based UIViewController.
    // Since this app is intended to be a SwiftUI demo app, I'm not going to switch to use UIHostingController (for now).
    //
    // If iOS has fixed this in 14.5 or later, we can enable this code. But, we will have to check the OS version in code,
    // or set the deployment target for the iOS version which finally fixes this bug.
    //
    // UPDATE 2: I tested with iOS15.7 and it is still broken with SwiftUI, at least when using a Scene-based Lifecycle.
    // So, if you need to do in-app pairing in a SwiftUI APP, we recommmend that you use the UIHostingController approach
    // as described above. Or, you can use UIKit to create a similar view instead of using SwiftUI.
    // If you require more information or assistance with this, please contact Tech Support.

    func pairPrintersByBluetoothMFi()
    {
        EAAccessoryManager.shared().showBluetoothAccessoryPicker(withNameFilter: nil) { (error) in
            // TODO: Continue implementing this completionHandler once Apple fixes the bug with the Picker.
            // Generally, this is only for errors anyway.
            // When user picks a device and pairs successfully, the EAAccessoryDidConnect notification should be fired.
            // We are already handling that. c.f. handleConnectNotification().
            if let error = error {
                switch error {
                case EABluetoothAccessoryPickerError.alreadyConnected:
                    break
                default:
                    break
                }
            }
        }
    }
#endif
    
} // struct BluetoothSearchResultView

#if DEBUG
struct BluetoothSearchResultView_EAAccessory_Previews: PreviewProvider {
    @State static var mfiSerialNumber = "ABC1234567890"
    
    static var previews: some View {
        BluetoothSearchResultView_EAAccessory(printersToSearch: SupportedModels.getArrayOfAllSupportedMFiModels(),
                                              mfiSerialNumber: $mfiSerialNumber)
    }
}
#endif

