//
//  WIFISearchResultView.swift
//  BMS_SwiftUI_Sample_v4_Basic
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
//  Created by Rob Roy
//  Copyright © 2020 Brother Mobile Solutions. All rights reserved.
//

import SwiftUI

struct WIFISearchResultView: View {
    @ObservedObject var brotherPrinter = BrotherPrinter.shared
    
    //*** Parameters that must be passed in by caller in init
    // Caller must pass in the ARRAY of printer models we want Search to find, as a FILTER.
    // * For a GENERIC search (e.g PrinterSearchTopView), it should contain *ALL* WIFI models the app supports.
    // * However, in some cases, you may only want to find a SPECIFIC model. So, the array should contain ONLY
    //   the 1 model you want to find.
    var printersToSearch:[String]
    var doSearchOnAppear:Bool // caller decides this

    // In case our "parent" view wants to display the result, it must define the @State var and pass it in.
    @Binding var wifiIPAddress:String
    
    //*** Vars that we define in here
    @State var discoveredPrinterList:[BRPtouchDeviceInfo] = [] // list of devices returned by search
    @State var selectedPrinter:BRPtouchDeviceInfo? = nil // used to check row and update printer model
    @State var infoText:String = ""
    @State var isSearching:Bool = false
    
    // SearchCoordinator
    // NOTE: A SwiftUI View cannot conform to an ObjC delegate protocol, which SDK requires with WIFI search.
    // So, we will define a "nested class" that conforms to this protocol to use for the search.
    // NOTE: It's necessary to persist this class from startSearch to didFinishSearch, so we need to store it in a var.
    // Since we must instantiate it with "self", we define this as a @State var.
    //
    // WARNING: Because WIFI search requires a "delegate" callback from SDK, if the user navigates back before the "didFinishSearch"
    // callback is called, the app will crash. So, we must disable the "back button" to prevent this.
    @State var coordinator:SearchCoordinator? = nil
    
    var body: some View {
        VStack{
            Spacer()
            Text(infoText)
                .background(Color.red)
            
            Spacer()
            List {
                Section(header: Text("Search Result List:"))
                {
                    // NOTE: "id:\.self" is required here if the array does not contain items that are "Identifiable"
                    // This indicates that each item is guaranteed to be unique.
                    // But, suppose there are multiple PJ-773 connected to same network. They all have strPrinterName = "PJ-773". Not unique!
                    // However, as the following article explains, ".self" means the WHOLE object, which will at least have a unique serialNumber.
                    // And, what actually happens is that Swift computes the HASH VALUE of the object to use as the identifier.
                    // So, this should be OK. TODO: Test this with multiple PJ-773 connected to same Infrastructure network.
                    // https://www.hackingwithswift.com/books/ios-swiftui/why-does-self-work-for-foreach
                    
                    ForEach(discoveredPrinterList, id:\.self) { discoveredPrinter in
                        Button(action: {
                            self.handleRowTap(deviceInfo: discoveredPrinter)
                            
                        })
                        {
                            // NOTE: For WIFI, the serialNumber will be best to use for unique identity (check mark)
                            // "strPrinterName" will NOT be unique. It seems that strPrinterName = strModelName.
                            ChecklistRow (title: discoveredPrinter.strSerialNumber,
                                          subtitle: discoveredPrinter.strPrinterName,
                                          isChecked: self.selectedPrinter?.strSerialNumber == discoveredPrinter.strSerialNumber)
                            .frame(height:20)
                        } // Button
                    } // ForEach
                } // Section header
            } // List
            .onAppear{
                if self.doSearchOnAppear {self.doSearchByWIFI()}
            }
            .navigationBarTitle(Text("Select WIFI Printer"), displayMode: .inline)
            .navigationBarItems( // add a search button to NavBar so user can redo the search, disable it while searching
                trailing: Button(action: {self.doSearchByWIFI()}) { Text("Search")}.disabled(self.isSearching)
            )
            // NOTE: backButton will NOT hide UNLESS you add a title and/or item to NavBar. Seems like a bug in SwiftUI.
            // And be sure to update GUI (via @State var) only on main thread, per usual.
            .navigationBarBackButtonHidden(self.isSearching)
            
        } // VStack
    } // body
    
    // SearchCoordinator
    // NOTE: A SwiftUI View cannot conform to an ObjC delegate protocol, which SDK requires with WIFI search.
    // So, we define this nested class to conform to the protocol to use for the search instead.
    //
    // NOTE: We don't actually need to define this View as UIViewRepresentable to handle this.
    // But, we can use the same Coordinator design pattern to handle it in a similar way.
    class SearchCoordinator: NSObject,BRPtouchNetworkDelegate {
        var parent: WIFISearchResultView
        
        init(_ parent: WIFISearchResultView) {
            self.parent = parent
        }
        
        // Start the search using SDK method
        func startSearchWiFiPrinter(printerNameArray:[String], searchTimeout:Int32) {
            
            // NOTE: We must tell SDK which printer model(s) we want to find.
            // If printerName(s) are not specified, then no printers will be found.
            // If only 1 model, you can just use printerName and a string instead of using printerNames with an array.
            // Alternatively, you can use the basic init and then call setPrinterName(s) afterward.
            // But, conceivably that could fail, which (I guess) is why manager is an "optional" when using the designated initializers.
            
            if let manager = BRPtouchNetworkManager(printerNames: printerNameArray)
            {
                manager.delegate = self
                manager.startSearch(searchTimeout)
            }
            else {
                print("ERROR: startSearchWiFiPrinter could not create manager.")
            }
        }
        
        // delegate callback from SDK with the result of the search
        func didFinishSearch(_ sender: Any!) {
            print("didFinishWIFISearch completed")
            
            // Update the "parent" View's GUI
            parent.infoText = "Search completed!"
            parent.isSearching = false
            
            guard let manager = sender as? BRPtouchNetworkManager else {
                print("didFinishSearch ERROR: invalid manager")
                
                return
            }
            guard let devices = manager.getPrinterNetInfo() else {
                print("didFinishSearch ERROR: invalid devices")
                
                return
            }
            
            // Update "parent" var with the discovered list of devices.
            parent.discoveredPrinterList = devices as? [BRPtouchDeviceInfo] ?? []
            
#if DEBUG
            // Print the found devices to the console.
            for deviceInfo in devices {
                if let deviceInfo = deviceInfo as? BRPtouchDeviceInfo {
                    print("Model: \(deviceInfo.strModelName ?? "unavailable"), IP Address: \(deviceInfo.strIPAddress ?? "unavailable")")
                }
            }
#endif
        } // didFinishSearch
        
    } // class SeearchCoordinator
    
    // get user selected printer fron the list
    func handleRowTap(deviceInfo:BRPtouchDeviceInfo)
    {
        selectedPrinter = deviceInfo
        // NOTE: refer to SDK user guide. BRLMPrinterClassifier is a utility class.
        // This method converts model name into A BRLMPrinterModel
        let sdkmodel:BRLMPrinterModel = BRLMPrinterClassifier.transferEnum(from: deviceInfo.strModelName)
        
        // Convert sdk model to one of our "supported" models.
        // If detected/selected printer is not one that our app supports, this will return nil.
        let selectedModel:SupportedModels? = SupportedModels.supportedModelFromBRLMPrinterModel(model:sdkmodel)
        
        if selectedModel != nil
        {
            // The binding var must be set so we can update the IPAddress TextField in WIFIChannelView
            // during user PrinterSetup, which also uses this same search result view.
            self.wifiIPAddress = deviceInfo.strIPAddress
            
            // This utility method assures we set these 2 things in the right order.
            self.brotherPrinter.setModelAndChannel(newModel: selectedModel!, newChannel: BRLMChannel(wifiIPAddress:deviceInfo.strIPAddress))
        }
    } // handleRowTap
    
    func doSearchByWIFI()
    {
        // Update the GUI and clear the search result list
        self.infoText = "Searching, please wait..."
        self.discoveredPrinterList = []
        self.isSearching = true
        
        // Start the search, using our "coordinator", which is needed to handle the SDK's "delegate" callback.
        // NOTE: WIFI search requires specifying the models you want to find.
        // Caller must setup this array, as sometimes we want to find all, other times we want to find only a specific model.
        self.coordinator = SearchCoordinator(self)
#if TEST_ONLY
        // NOTE: Using "Brother" here does work as a filter to find any printer with "Brother" in the name.
        // However, this may find printers NOT supported by this SDK. So, this is not recommended.
        self.coordinator?.startSearchWiFiPrinter(printerNameArray: ["Brother"], searchTimeout: 5)
#else
        // This is recommended way, using the "printersToSearch" array that is passed in
        self.coordinator?.startSearchWiFiPrinter(printerNameArray: self.printersToSearch, searchTimeout: 5)
#endif
    } // doSearchByWIFI
    
    
} // struct view


#if DEBUG
struct SearchResultView_Previews: PreviewProvider {
    @State static var ipAddress = "12.34.56.789"
    
    static var previews: some View {
        NavigationView {
            WIFISearchResultView(printersToSearch: SupportedModels.getArrayOfAllSupportedWIFIModels(),
                                 doSearchOnAppear: true,
                                 wifiIPAddress: $ipAddress)
        }
        
        
        
    }
    
} // preview
#endif
