//
//  PrintHandler.swift
//  BMS_SwiftUI_Sample_v4_Basic
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
//  Created by Rob Roy
//  Copyright © 2020 Brother Mobile Solutions. All rights reserved.
//

import Foundation

// Super class for common methods used by the other PrintHandlers
class PrintHandler {
    
    // Need to store this outside of each printing function to allow cancelPrinting
    var printerDriver:BRLMPrinterDriver? = nil
    
    // *********************************************************
    // cancelPrinting
    // -> Allow user to cancel printing via a ProgressView
    // *********************************************************
    func cancelPrinting()
    {
        self.printerDriver?.cancelPrinting()
    }
    
    // *********************************************************
    // checkPrinterStatus
    // -> Ask printer to return its status response. If received, evaluate it.
    //
    // NOTE: This can only be sent at the BEGINNING or END of a job.
    // Not all fields of the "rawData" in the status response will be valid at these times. Many fields are used by SDK internally.
    // Review the Raster Command Reference Guide for your desired printer model to understand the response returned.
    //
    // UPDATE: SDK 4.5.1 has added a lot of new info to BRLMPrinterStatus class to help evaluate the status response.
    // It may not be necessary anymore to look at the "rawData" to understand the response.
    // * "rawData" contains the raw status response data.
    //   - This replaces "ptStatus" from older SDK (which is now "deprecated")
    //   - Refer to appropriate "Raster Command Reference Guide" for details.
    //   - But, the SDK now evaluates this and provides most of the common issues in "errorCode".
    //   - So, it is not necessary to evaluate this yourself, in most cases.
    // * "model" contains the connected printer model (same as older SDKs)
    // * "errorCode" can be used to detect common issues before attempting to print
    // * "batteryStatus" can be used to obtain detailed battery information
    //   - This may be useful in a different function to display battery info
    // * "mediaInfo" can provide details about the installed media (not valid with all printer models)
    //    - This may be especially useful for QL and PT models to AUTOMATICALLY DETECT the currently installed labelSize,
    //      which MUST be set correctly into BRLM[QL,PT]PrintSettings.labelSize when printing PDF or IMAGE.
    //
    // PARAMETERS:
    // * printerDriver: created by successful call to "openChannel", allows access to functions which communicate to the printer
    // * expectedModel: used to assure we are communicating with the same printer model we are expecting
    // * printSettings:
    //    - "nil" for PRN data
    //    - for PDF/Image data, should be of correct type matching the "expectedModel" (e.g. BRLMQLPrintSettings for QL models)
    //    - Used for QL and PT printers ONLY: "printSettings.labelSize" will be REPLACED with the detected size.
    //
    // RETURNS:
    // * "nil" if printer checks out OK.
    // * an error string if wrong model is connected or some other problem detected by the printer status response.
    // NOTE: We return String so the View can display this in an Alert.
    // *********************************************************
    func checkPrinterStatus(printerDriver:BRLMPrinterDriver, expectedModel:BRLMPrinterModel, printSettings:BRLMPrintSettingsProtocol?) -> String?
    {
        print("checkPrinterStatus")
        
        // Ask printer for status
        let statusResult:BRLMGetPrinterStatusResult = printerDriver.getPrinterStatus()
        if statusResult.error.code != .noError
        {
            var errorMsg = "checkPrinterStatus:\nError getting status:\n"
            
            // NOTE: code is of type BRLMGetStatusErrorCode
            switch statusResult.error.code
            {
            case .noError:
                break // won't get here per if, ignore
            case .timeout:
                errorMsg += "TIMEOUT"
            case .printerNotFound:
                errorMsg += "Printer Not Found"
            @unknown default:
                errorMsg += "Unknown Error. Code: \(statusResult.error.code)"
            }
            
            print(errorMsg)
            
            return errorMsg
        } // if error getting status
        else
        {
            // status was received OK. Now evaluate it.
            
            // Assure statusResult.status is not nil before continuing
            // NOTE: statusResult.status is of type BRLMPrinterStatus
            guard let printerStatus = statusResult.status else {
                let errorMsg = "checkPrinterStatus: Error - statusResult contains empty status"
                print(errorMsg)
                
                return errorMsg
            }
            
            //*** If status has an errorCode, convert and return a string
            // As of SDK 4.5.1, BRLMPrinterStatus will evaluate the status response and provide an "errorCode"
            // NOTE: errorCode is of type BRLMPrinterStatusErrorCode
            if printerStatus.errorCode != .noError
            {
                // Create a string from the errorCode
                
                // NOTE: If you wish to ignore any of these and print anyway, return nil instead of generating a string.
                // Or, allow the calling function to continue even if we return an errorMsg string.
                // As mentioned above, some of these may not be valid until you attempt to print, depending on the printer model.
                
                var errorMsg = "checkPrinterStatus:\nPrinter Status Issue:\n"
                
                switch printerStatus.errorCode
                {
                case .noError:
                    break // won't get here per if, ignore
                case .noPaper:
                    // NOTE: When testing this with RJ-32, RJ-42, and PJ8 models, this condition is NOT being caught before
                    // printing. It's not in the "rawData" of the status response either.
                    // This might be one of those things that isn't valid in status response unless you are printing, at least
                    // for some printer models.
                    errorMsg += "No Paper"
                case .coverOpen:
                    errorMsg += "Cover Open"
                case .busy:
                    errorMsg += "Busy"
                case .paperJam:
                    errorMsg += "Paper Jam"
                case .highVoltageAdapter:
                    errorMsg += "High Voltage Adapter"
                case .batteryEmpty:
                    errorMsg += "Battery Empty"
                case .batteryTrouble:
                    errorMsg += "Battery Trouble"
                case .tubeNotDetected:
                    errorMsg += "Tube Not Detected"
                case .systemError:
                    errorMsg += "System Error"
                case .anotherError:
                    errorMsg += "Another Error"
                @unknown default:
                    errorMsg += "Unknown Error. Code: \(printerStatus.errorCode)"
                }
                
                return errorMsg
            } // if printerStatus.errorCode != .noError
            else {
                // No errors with status response.
                // Now, we can use the BRLMPrinterStatus object to check some other things...
                
                //*** 1. Assure model connected matches the expectedModel.
                // This assures that we won't send the wrong print data for the model.
                //
                // NOTE: With PDF/IMAGE printing, SDK will NOT allow printing anyway, so this is NOT necessary.
                // However, for PRN data, SDK doesn't check since PrintSettings are not used with PRN.
                // So, this can serve as a "gate" to prevent using the app with compatible non-certified models.
                //
                // NOTE: The easiest way to generate this problem is to use Wireless Direct with 2 different Brother models.
                //  - e.g. PJ-883 and RJ-4250WB
                //  - Connect WIFI to one model, then search and select it in this app. This becomes the "expectedModel".
                //  - Then, connect WIFI to the other model and try to print.
                //  - Since IP Address will be the same for each, openChannel will likely succeed.
                //  - So, we can prevent sending the wrong PRN data to the other model with this check.
                if expectedModel != printerStatus.model
                {
                    let errorMsg = "checkPrinterStatus: Error - wrong model connected"
                    print(errorMsg)
                    
                    return errorMsg
                }
                
                // Now, use new feature in SDK 4.5.1...

                //*** 2. If QL or PT model, detect the currently installed label.
                // - If successful, UPDATE printSettings.labelSize to contain the detected labelSize (as a v4 enum value)
                // - This is required when printing PDF or IMAGE, otherwise SDK will FAIL when printing if the wrong labelSize is specified.
                //
                // NOTE: Don't do this when printSettings are nil (i.e. for PRN jobs)
                if printSettings != nil
                {
                    // mediaInfo will NOT be valid with all models. Must check for nil.
                    // if valid, unwrap the optional into local var
                    guard let mediaInfo = printerStatus.mediaInfo else {
                        // Not an error situation. Just not a QL or PT model so we can return.
                        return nil
                    }
                    
                    // NOTE: The getQLlabelSize and getPTLabelSize functions in BRLMMediaInfo will only "succeed"
                    // if the "connected" printer is a model in that family.
                    
                    var succeeded:Bool = false
                    
                    let qlSize = mediaInfo.getQLLabelSize(&succeeded)
                    if succeeded {
                        // Assure printSettings is QL type before trying to set labelSize, to avoid potential crash.
                        // Not all BRLMxxPrintSettings classes have a labelSize property.
                        if printSettings is BRLMQLPrintSettings {
                            let qlSettings = printSettings as! BRLMQLPrintSettings
                            
                            qlSettings.labelSize = qlSize
                        }
                        else {
                            // log to console only, so as not to prevent printing if everything else is OK.
                            print("checkPrinterStatus: WARNING: printSettings type does not match connected QL printer.")
                            return nil // no error being reported, but maybe still a problem
                        }
                    }
                    
                    let ptSize = mediaInfo.getPTLabelSize(&succeeded)
                    if succeeded {
                        // Assure printSettings is PT type before trying to set labelSize, to avoid potential crash.
                        // Not all BRLMxxPrintSettings classes have a labelSize property.
                        if printSettings is BRLMPTPrintSettings {
                            let ptSettings = printSettings as! BRLMPTPrintSettings
                            
                            ptSettings.labelSize = ptSize
                        }
                        else {
                            // log to console only, so as not to prevent printing if everything else is OK.
                            print("checkPrinterStatus: WARNING: printSettings type does not match connected PT printer.")
                            return nil // no error being reported, but maybe still a problem
                        }
                    }
                } // if printSettings not nil
            } // else no errors with status response
        } // else statusResult received OK
        
        return nil // nil means all OK
        
    } // checkPrinterStatus
    
    // *********************************************************
    // validatePrintSettings
    // -> This can be used primarily for DEBUG, to get more information about the cause of an error at print-time
    // if it is related to print settings.
    //
    // RETURNS:
    // * nil if no issues
    // * error string if there are any significant issues
    // *********************************************************
    func validatePrintSettings(printSettings:BRLMPrintSettingsProtocol) -> String?
    {
        //*** Generate a BRLMValidatePrintSettingsReport by allowing SDK to evaluate the printSettings.
        // NOTE:
        // * if report.errorCount > 0, printing will fail. But, if we proceed to print anyway, we can get an error code as well and handle that error.
        // * if report.warningCount > 0, there is an issue with settings, but the SDK will modify settings internally to print the job anyway.
        // * if report.noticeCount > 0, there is an issue with settings, but it is non-fatal and SDK can print without any internal modifications.
        // * In either case, you should investigate the settings and modify them as necessary.
        let report:BRLMValidatePrintSettingsReport = BRLMValidatePrintSettings.validate(printSettings)
        print("validatePrintSettings report:\n\(report.description)")
        
        if report.errorCount > 0 || report.warningCount > 0 || report.noticeCount > 0
        {
            // Allow caller to know there's a potential issue and add this to an alert if desired.
            return report.description
        }
        
        return nil // no issues
    } // validatePrintSettings
    
    
    // *********************************************************
    // stringFromOpenChannelError
    // -> Produce an error string from the result of calling the BRLMPrinterDriverGenerator's open channel API
    //
    // RETURNS:
    // * nil if no issues
    // * error string if there are any issues
    // *********************************************************
    func stringFromOpenChannelError(openChannelError:BRLMOpenChannelError, callingFunction:String) -> String
    {
        // NOTE: BRLMOpenChannelError.error currently ONLY has "code", it does NOT provide a "description" like BRLMPrintError does (for now).
        // So, we will add our own description for each "code" here.
        
        var errorMsg = "\(callingFunction): Open Channel ERROR: "
        
        switch openChannelError.code {
        case .noError:
            // Shouldn't call this in this case, but handle it anyway.
            errorMsg = "\(callingFunction): Open Channel: SUCCESS"
            
        case .openStreamFailure:
            errorMsg = errorMsg + "openStreamFailure"
            
        case .timeout:
            errorMsg = errorMsg + "TIMEOUT"
            
        @unknown default:
            errorMsg = errorMsg + "unknown default"
            
        }
        
        // NOTE: Since BRLMOpenChannelError is a subclass of BRLMError, it could potentially provide an "errorRecoverySuggestion".
        // If so, let's add it to the message. So far, it seems that nothing is available here. Must check for nil.
        if openChannelError.errorRecoverySuggestion != nil {
            errorMsg += "\nRecovery: \(openChannelError.errorRecoverySuggestion!)"
        }
        
        return errorMsg
    }
    
    // *********************************************************
    // stringFromPrintError
    // -> Produce an error string from the result of calling any printing API
    //
    // RETURNS:
    // * nil if no issues
    // * error string if there are any issues
    // *********************************************************
    func stringFromPrintError(printError:BRLMPrintError, callingFunction:String) -> String
    {
#if SDK_BEFORE_461
        // NOTE: SDK provides "description" for this. So, let's use it, unless we want to make our own description.
        var errorMsg = "\(callingFunction): ERROR: \(printError.description)"
        
        // NOTE: Since BRLMPrintError is a subclass of BRLMError, it could potentially provide an "errorRecoverySuggestion".
        // If so, let's add it to the message. So far, it seems that nothing is available here. Must check for nil.
        if printError.errorRecoverySuggestion != nil {
            errorMsg += "\nRecovery: \(printError.errorRecoverySuggestion!)"
        }
#else
        // IMPORTANT: SDK 4.6.1 changed error handling.
        // BRLMPrintError no longer inherits from BRLMError (which is now deprecated).
        // It now inherits from NSObject instead.
        //
        // To handle the changes:
        // 1. BRLMError.description -> BRLMPrintError.errorDescription
        // 2. If you need more detailed information to understand the error code (or description) that is returned,
        // then use the new "allLogs" **ARRAY** to get this information.
        var errorMsg = "\(callingFunction): ERROR: \(printError.errorDescription)"
        
        printError.allLogs.forEach { log in
            // write details to console.
            // Do this for all builds, in case console information may be useful in RELEASE.
            print(log)
#if DEBUG
            // If desired, add these details to the return string
            //
            // NOTE: It is NOT recommended to show this to End User, for the following reasons:
            // 1. It produces REDUNDANT information when concatenated with printError.errorDescription,
            //    because this will contain the SAME info as the LAST message in log[n] with the
            //    HIGHEST RANK (i.e. rank: Error > Warning > Notice)
            // 2. Showing End User these log details may confuse them.
            // 3. This log information is generally intended for ISV to DEBUG situations where the error code and
            //    printError.errorDescription may be unclear.
            //
            // So, we will add this to the string for the GUI display ONLY for DEBUG build!!
            errorMsg += "\n\nDetail: \(log)"
#endif
            
        }
#endif
        
        // NOTE: In some cases, it may also be useful for End User (i.e. Release build) to concatenate the result of
        // "validatePrintSettings" (see function in this class) to this string, as our printing code demonstrates.
        // i.e. printError.errorDescription may not always be useful for End User as worded, even though the SDK tries
        // to put the most helpful error message here. The validate message may provide more a more useful description.
        // c.f. each xxxPrintHandler function for how it does this.
        
        return errorMsg
    }
    
} // class PrintHandler
